@@ -11,17 +11,15 @@
 #ifndef __AUDACITY_SPLASH_DLG__
 #define __AUDACITY_SPLASH_DLG__
 
-#include <wx/dialog.h>
+#include "MemoryX.h"
+#include "widgets/wxPanelWrapper.h" // to inherit
 
-class wxBoxSizer;
-class wxStaticBitmap;
 class wxBitmap;
 class ShuttleGui;
 class AudacityProject;
-class wxCheckbox;
 class HtmlWindow;
 
-class SplashDialog:public wxDialog {
+class SplashDialog final : public wxDialogWrapper {
    DECLARE_DYNAMIC_CLASS(SplashDialog)
 public:
    SplashDialog(wxWindow * parent);
@@ -37,8 +35,7 @@ private:
    void OnDontShow( wxCommandEvent & Evt );
 
    HtmlWindow * mpHtml;
-   wxStaticBitmap* m_pIcon;
-   wxBitmap* m_pLogo; //vvv
+   std::unique_ptr<wxBitmap> m_pLogo; //vvv
    static SplashDialog * pSelf;
 };
 
