@@ -16,17 +16,21 @@ The summary is eventually computed and written to a file in a background thread.
 
 *//*******************************************************************/
 
-#include <float.h>
+#include "../Audacity.h"
 #include "ODDecodeBlockFile.h"
 
+#include <float.h>
+
 #include <wx/utils.h>
-#include <wx/wxchar.h>
+#include <wx/wxcrtvararg.h>
 #include <wx/log.h>
 #include <wx/thread.h>
 #include <sndfile.h>
 
+#include "../FileException.h"
 #include "../FileFormats.h"
 #include "../Internat.h"
+#include "NotYetAvailableException.h"
 
 const int bheaderTagLen = 20;
 char bheaderTag[bheaderTagLen + 1] = "AudacityBlockFile112";
@@ -34,33 +38,43 @@ char bheaderTag[bheaderTagLen + 1] = "AudacityBlockFile112";
 
 
    /// Create a disk file and write summary and sample data to it
-ODDecodeBlockFile::ODDecodeBlockFile(wxFileName baseFileName,wxFileName audioFileName, sampleCount aliasStart,
-                     sampleCount aliasLen, int aliasChannel,unsigned int decodeType):
-   SimpleBlockFile(baseFileName,NULL,aliasLen,floatSample,true,true), //floatSample has no effect.  last two bools - bypass writing of blockfile and cache
+ODDecodeBlockFile::ODDecodeBlockFile(wxFileNameWrapper &&baseFileName, wxFileNameWrapper &&audioFileName, sampleCount aliasStart,
+                     size_t aliasLen, int aliasChannel,unsigned int decodeType)
+try
+   : SimpleBlockFile{ std::move(baseFileName),
+                    NULL, aliasLen, floatSample, true, true },
+   //floatSample has no effect.  last two bools - bypass writing of blockfile and cache
 
    mType(decodeType),
    mAliasStart(aliasStart),
    mAliasChannel(aliasChannel)
 {
    mDecoder = NULL;
-   mDataAvailable=false;
-   mAudioFileName = audioFileName;
+   mAudioFileName = std::move(audioFileName);
    mFormat = int16Sample;
 }
+catch ( const FileException & WXUNUSED(e) )
+{
+   // The constructor SimpleBlockFile sometimes throws this,
+   // but it never will for the arguments that were passed to it here.
+   // So add a catch for completeness, but just assert that this won't happen.
+   wxASSERT(false);
+   throw;
+}
 
 /// Create the memory structure to refer to the given block file
-ODDecodeBlockFile::ODDecodeBlockFile(wxFileName existingFile, wxFileName audioFileName, sampleCount aliasStart,
-                     sampleCount aliasLen, int aliasChannel, unsigned int decodeType,
+ODDecodeBlockFile::ODDecodeBlockFile(wxFileNameWrapper &&existingFile, wxFileNameWrapper &&audioFileName, sampleCount aliasStart,
+                     size_t aliasLen, int aliasChannel, unsigned int decodeType,
                    float min, float max, float rms, bool dataAvailable):
-   SimpleBlockFile(existingFile,aliasLen,min,max,rms),
+   SimpleBlockFile{ std::move(existingFile), aliasLen, min, max, rms },
 
    mType(decodeType),
+   mDataAvailable( dataAvailable ),
    mAliasStart(aliasStart),
    mAliasChannel(aliasChannel)
 {
    mDecoder = NULL;
-   mDataAvailable=dataAvailable;
-   mAudioFileName = audioFileName;
+   mAudioFileName = std::move(audioFileName);
    mFormat = int16Sample;
 }
 
@@ -73,7 +87,7 @@ ODDecodeBlockFile::~ODDecodeBlockFile()
 
 
 //Check to see if we have the file for these calls.
-wxLongLong ODDecodeBlockFile::GetSpaceUsage()
+auto ODDecodeBlockFile::GetSpaceUsage() const -> DiskByteCount
 {
    if(IsSummaryAvailable())
    {
@@ -88,42 +102,49 @@ wxLongLong ODDecodeBlockFile::GetSpaceUsage()
 
 
 /// Gets extreme values for the specified region
-void ODDecodeBlockFile::GetMinMax(sampleCount start, sampleCount len,
-                          float *outMin, float *outMax, float *outRMS)
+auto ODDecodeBlockFile::GetMinMaxRMS(
+   size_t start, size_t len, bool mayThrow) const -> MinMaxRMS
 {
    if(IsSummaryAvailable())
    {
-      SimpleBlockFile::GetMinMax(start,len,outMin,outMax,outRMS);
+      return SimpleBlockFile::GetMinMaxRMS(start, len, mayThrow);
    }
    else
    {
+      if (mayThrow)
+         throw NotYetAvailableException{ mAudioFileName };
+
       //fake values.  These values are used usually for normalization and amplifying, so we want
       //the max to be maximal and the min to be minimal
-      *outMin = -1.0;
-      *outMax = 1.0;
-      *outRMS = (float)0.707;//sin with amp of 1 rms
+      return {
+         -1.0f, 1.0f, 0.707f //sin with amp of 1 rms
+      };
    }
 }
 
 /// Gets extreme values for the entire block
-void ODDecodeBlockFile::GetMinMax(float *outMin, float *outMax, float *outRMS)
+auto ODDecodeBlockFile::GetMinMaxRMS(bool mayThrow) const -> MinMaxRMS
 {
   if(IsSummaryAvailable())
    {
-      SimpleBlockFile::GetMinMax(outMin,outMax,outRMS);
+      return SimpleBlockFile::GetMinMaxRMS(mayThrow);
    }
    else
    {
+      if (mayThrow)
+         throw NotYetAvailableException{ mAudioFileName };
+
       //fake values.  These values are used usually for normalization and amplifying, so we want
       //the max to be maximal and the min to be minimal
-      *outMin = -1.0;
-      *outMax = 1.0;
-      *outRMS = (float)0.707;//sin with amp of 1 rms
+      return {
+         -1.0f, 1.0f, 0.707f //sin with amp of 1 rms
+      };
    }
 }
 
 /// Returns the 256 byte summary data block
-bool ODDecodeBlockFile::Read256(float *buffer, sampleCount start, sampleCount len)
+/// Fill with zeroes and return false if data are unavailable for any reason.
+bool ODDecodeBlockFile::Read256(float *buffer, size_t start, size_t len)
 {
    if(IsSummaryAvailable())
    {
@@ -131,14 +152,14 @@ bool ODDecodeBlockFile::Read256(float *buffer, sampleCount start, sampleCount le
    }
    else
    {
-      //this should not be reached (client should check IsSummaryAvailable()==true before this.
-      buffer = NULL;
-      return true;
+      ClearSamples((samplePtr)buffer, floatSample, 0, len);
+      return false;
    }
 }
 
 /// Returns the 64K summary data block
-bool ODDecodeBlockFile::Read64K(float *buffer, sampleCount start, sampleCount len)
+/// Fill with zeroes and return false if data are unavailable for any reason.
+bool ODDecodeBlockFile::Read64K(float *buffer, size_t start, size_t len)
 {
    if(IsSummaryAvailable())
    {
@@ -146,39 +167,37 @@ bool ODDecodeBlockFile::Read64K(float *buffer, sampleCount start, sampleCount le
    }
    else
    {
-      //this should not be reached (client should check IsSummaryAvailable()==true before this.
-      return true;
+      ClearSamples((samplePtr)buffer, floatSample, 0, len);
+      return false;
    }
 }
 
 /// If the summary has been computed,
-/// Construct a new PCMAliasBlockFile based on this one.
-/// otherwise construct an ODPCMAliasBlockFile that still needs to be computed.
+/// Construct a NEW SimpleBlockFile based on this one.
+/// otherwise construct an ODDecodeBlockFile that still needs to be computed.
 /// @param newFileName The filename to copy the summary data to.
-BlockFile *ODDecodeBlockFile::Copy(wxFileName newFileName)
+BlockFilePtr ODDecodeBlockFile::Copy(wxFileNameWrapper &&newFileName)
 {
-   BlockFile *newBlockFile;
+   BlockFilePtr newBlockFile;
 
    //mAliasedFile can change so we lock readdatamutex, which is responsible for it.
-   LockRead();
+   auto locker = LockForRead();
    if(IsSummaryAvailable())
    {
       //create a simpleblockfile, because once it has the summary it is a simpleblockfile for all intents an purposes
-      newBlockFile  = SimpleBlockFile::Copy(newFileName) ;
+      newBlockFile  = SimpleBlockFile::Copy(std::move(newFileName)) ;
    }
    else
    {
       //Summary File might exist in this case, but it probably (99.999% of the time) won't.
-      newBlockFile  = new ODDecodeBlockFile(newFileName,
-                                                   mAudioFileName, mAliasStart,
-                                                   mLen, mAliasChannel, mType,
-                                                   mMin, mMax, mRMS,IsSummaryAvailable());
-      //The client code will need to schedule this blockfile for OD decoding if it is going to a new track.
+      newBlockFile = make_blockfile<ODDecodeBlockFile>
+         (std::move(newFileName), wxFileNameWrapper{mAudioFileName},
+          mAliasStart, mLen, mAliasChannel, mType,
+          mMin, mMax, mRMS, IsSummaryAvailable());
+      //The client code will need to schedule this blockfile for OD decoding if it is going to a NEW track.
       //It can do this by checking for IsDataAvailable()==false.
    }
 
-   UnlockRead();
-
    return newBlockFile;
 }
 
@@ -188,8 +207,9 @@ BlockFile *ODDecodeBlockFile::Copy(wxFileName newFileName)
 /// Most notably, the summaryfile attribute refers to a file that does not yet, so when the project file is read back in
 /// and this object reconstructed, it needs to avoid trying to open it as well as schedule itself for OD loading
 void ODDecodeBlockFile::SaveXML(XMLWriter &xmlFile)
+// may throw
 {
-   LockRead();
+   auto locker = LockForRead();
    if(IsSummaryAvailable())
    {
       SimpleBlockFile::SaveXML(xmlFile);
@@ -197,35 +217,37 @@ void ODDecodeBlockFile::SaveXML(XMLWriter &xmlFile)
    else
    {
       xmlFile.StartTag(wxT("oddecodeblockfile"));
-       //unlock to prevent deadlock and resume lock after.
-      UnlockRead();
-      mFileNameMutex.Lock();
-      xmlFile.WriteAttr(wxT("summaryfile"), mFileName.GetFullName());
-      mFileNameMutex.Unlock();
-      LockRead();
+      {
+         //unlock to prevent deadlock and resume lock after.
+         auto suspension = locker.Suspend();
+         ODLocker locker2{ &mFileNameMutex };
+         xmlFile.WriteAttr(wxT("summaryfile"), mFileName.GetFullName());
+      }
       xmlFile.WriteAttr(wxT("audiofile"), mAudioFileName.GetFullPath());
-      xmlFile.WriteAttr(wxT("aliasstart"), mAliasStart);
+      xmlFile.WriteAttr(wxT("aliasstart"),
+                        mAliasStart.as_long_long());
       xmlFile.WriteAttr(wxT("aliaslen"), mLen);
       xmlFile.WriteAttr(wxT("aliaschannel"), mAliasChannel);
       xmlFile.WriteAttr(wxT("decodetype"), (size_t)mType);
 
       xmlFile.EndTag(wxT("oddecodeblockfile"));
    }
-   UnlockRead();
 }
 
-/// Constructs a ODPCMAliasBlockFile from the xml output of WriteXML.
-/// Also schedules the ODPCMAliasBlockFile for OD loading.
+/// Constructs a ODDecodeBlockFile from the xml output of WriteXML.
+/// Also schedules the ODDecodeBlockFile for OD loading.
 // BuildFromXML methods should always return a BlockFile, not NULL,
 // even if the result is flawed (e.g., refers to nonexistent file),
 // as testing will be done in DirManager::ProjectFSCK().
-BlockFile *ODDecodeBlockFile::BuildFromXML(DirManager &dm, const wxChar **attrs)
+BlockFilePtr ODDecodeBlockFile::BuildFromXML(DirManager &dm, const wxChar **attrs)
 {
-   wxFileName summaryFileName;
-   wxFileName audioFileName;
-   sampleCount aliasStart=0, aliasLen=0;
+   wxFileNameWrapper summaryFileName;
+   wxFileNameWrapper audioFileName;
+   sampleCount aliasStart = 0;
+   size_t aliasLen = 0;
    int aliasChannel=0;
    long nValue;
+   long long nnValue;
    unsigned int   decodeType=0;
 
    while(*attrs)
@@ -239,7 +261,7 @@ BlockFile *ODDecodeBlockFile::BuildFromXML(DirManager &dm, const wxChar **attrs)
       if (!wxStricmp(attr, wxT("summaryfile")) &&
             // Can't use XMLValueChecker::IsGoodFileName here, but do part of its test.
             XMLValueChecker::IsGoodFileString(strValue) &&
-            (strValue.Length() + 1 + dm.GetProjectDataDir().Length() <= PLATFORM_MAX_PATH))
+            (strValue.length() + 1 + dm.GetProjectDataDir().length() <= PLATFORM_MAX_PATH))
       {
          if (!dm.AssignFile(summaryFileName, strValue, false))
             // Make sure summaryFileName is back to uninitialized state so we can detect problem later.
@@ -258,11 +280,15 @@ BlockFile *ODDecodeBlockFile::BuildFromXML(DirManager &dm, const wxChar **attrs)
             // but we want to keep the reference to the file because it's a good path string.
             audioFileName.Assign(strValue);
       }
+      else if ( !wxStricmp(attr, wxT("aliasstart")) )
+      {
+         if (XMLValueChecker::IsGoodInt64(strValue) &&
+             strValue.ToLongLong(&nnValue) && (nnValue >= 0))
+            aliasStart = nnValue;
+      }
       else if (XMLValueChecker::IsGoodInt(strValue) && strValue.ToLong(&nValue))
       {  // integer parameters
-         if (!wxStricmp(attr, wxT("aliasstart")) && (nValue >= 0))
-            aliasStart = nValue;
-         else if (!wxStricmp(attr, wxT("aliaslen")) && (nValue >= 0))
+         if (!wxStricmp(attr, wxT("aliaslen")) && (nValue >= 0))
             aliasLen = nValue;
          else if (!wxStricmp(attr, wxT("aliaschannel")) && XMLValueChecker::IsValidChannel(aliasChannel))
             aliasChannel = nValue;
@@ -271,10 +297,9 @@ BlockFile *ODDecodeBlockFile::BuildFromXML(DirManager &dm, const wxChar **attrs)
       }
    }
 
-   return new ODDecodeBlockFile(summaryFileName, audioFileName,
-                                aliasStart, aliasLen, aliasChannel,decodeType,
-                                0,0,0, false);
-
+   return make_blockfile<ODDecodeBlockFile>
+      (std::move(summaryFileName), std::move(audioFileName),
+       aliasStart, aliasLen, aliasChannel, decodeType, 0, 0, 0, false);
 }
 
 
@@ -287,19 +312,16 @@ void ODDecodeBlockFile::Recover(void)
    }
 }
 
-bool ODDecodeBlockFile::IsSummaryAvailable()
+bool ODDecodeBlockFile::IsSummaryAvailable() const
 {
    return IsDataAvailable();
 }
 
-bool ODDecodeBlockFile::IsDataAvailable()
+bool ODDecodeBlockFile::IsDataAvailable() const
 {
-   bool retval;
-   mDataAvailableMutex.Lock();
-   retval= mDataAvailable;
-   mDataAvailableMutex.Unlock();
-   return retval;
+   return mDataAvailable != 0;
 }
+
 /// Write the summary to disk, using the derived ReadData() to get the data
 /// Here, the decoder ODTask associated with this file must fetch the samples with
 /// the ODDecodeTask::Decode() method.
@@ -308,72 +330,60 @@ int ODDecodeBlockFile::WriteODDecodeBlockFile()
 
    // To build the summary data, call ReadData (implemented by the
    // derived classes) to get the sample data
-   samplePtr sampleData;// = NewSamples(mLen, floatSample);
+   SampleBuffer sampleData;// = NewSamples(mLen, floatSample);
    int ret;
-   //use the decoder here.
-   mDecoderMutex.Lock();
 
-   if(!mDecoder)
    {
-      mDecoderMutex.Unlock();
-      return -1;
+      //use the decoder here.
+      ODLocker locker{ &mDecoderMutex };
+
+      if(!mDecoder)
+         return -1;
+
+      //sampleData and mFormat are set by the decoder.
+      ret = mDecoder->Decode(sampleData, mFormat, mAliasStart, mLen, mAliasChannel);
+
+      if(ret < 0) {
+         wxPrintf("ODDecodeBlockFile Decode failure\n");
+         return ret; //failure
+      }
    }
 
-
-   //sampleData and mFormat are set by the decoder.
-   ret = mDecoder->Decode(sampleData, mFormat, mAliasStart, mLen, mAliasChannel);
-
-   mDecoderMutex.Unlock();
-   if(ret < 0) {
-      printf("ODDecodeBlockFile Decode failure\n");
-      return ret; //failure
+   {
+      //the summary is also calculated here.
+      ODLocker locker{ &mFileNameMutex };
+      //TODO: we may need to write a version of WriteSimpleBlockFile that uses threadsafe FILE vs wxFile
+      bool bSuccess =
+         WriteSimpleBlockFile(
+                              sampleData.ptr(),
+                              mLen,
+                              mFormat,
+                              NULL);
+      if ( !bSuccess )
+         return -1;
    }
 
-   //the summary is also calculated here.
-   mFileNameMutex.Lock();
-   //TODO: we may need to write a version of WriteSimpleBlockFile that uses threadsafe FILE vs wxFile
-   bool bSuccess =
-      WriteSimpleBlockFile(
-         sampleData,
-         mLen,
-         mFormat,
-         NULL);//summaryData);
-   wxASSERT(bSuccess); // TODO: Handle failure here by alert to user and undo partial op.
-
-   mFileNameMutex.Unlock();
-
-   DeleteSamples(sampleData);
-//   delete [] (char *) summaryData;
-
-
-   mDataAvailableMutex.Lock();
-   mDataAvailable=true;
-   mDataAvailableMutex.Unlock();
+   wxAtomicInc( mDataAvailable );
 
    return ret;
 }
 
 ///sets the file name the summary info will be saved in.  threadsafe.
-void ODDecodeBlockFile::SetFileName(wxFileName &name)
+void ODDecodeBlockFile::SetFileName(wxFileNameWrapper &&name)
 {
    mFileNameMutex.Lock();
-   mFileName=name;
+   mFileName=std::move(name);
 /* mchinen oct 9 2009 don't think we need the char* but leaving it in for now just as a reminder that we might
    if wxFileName isn't threadsafe.
-   delete [] mFileNameChar;
-   mFileNameChar = new char[strlen(mFileName.GetFullPath().mb_str(wxConvUTF8))+1];
-   strcpy(mFileNameChar,mFileName.GetFullPath().mb_str(wxConvUTF8)); */
+   mFileNameChar.reinit(strlen(mFileName.GetFullPath().mb_str(wxConvUTF8))+1);
+   strcpy(mFileNameChar.get(), mFileName.GetFullPath().mb_str(wxConvUTF8)); */
    mFileNameMutex.Unlock();
 }
 
 ///sets the file name the summary info will be saved in.  threadsafe.
-wxFileName ODDecodeBlockFile::GetFileName()
+auto ODDecodeBlockFile::GetFileName() const -> GetFileNameResult
 {
-   wxFileName name;
-   mFileNameMutex.Lock();
-   name = mFileName;
-   mFileNameMutex.Unlock();
-   return name;
+   return { mFileName, ODLocker{ &mFileNameMutex } };
 }
 
 /// A thread-safe version of CalcSummary.  BlockFile::CalcSummary
@@ -386,23 +396,25 @@ wxFileName ODDecodeBlockFile::GetFileName()
 /// This method also has the side effect of setting the mMin, mMax,
 /// and mRMS members of this class.
 ///
-/// Unlike BlockFile's implementation You SHOULD delete the returned buffer.
+/// Unlike BlockFile's implementation You SHOULD DELETE the returned buffer.
 /// this is protected so it shouldn't be hard to deal with - just override
 /// all BlockFile methods that use this method.
 ///
 /// @param buffer A buffer containing the sample data to be analyzed
 /// @param len    The length of the sample data
 /// @param format The format of the sample data.
-void *ODDecodeBlockFile::CalcSummary(samplePtr buffer, sampleCount len,
-                             sampleFormat format)
+void *ODDecodeBlockFile::CalcSummary(samplePtr buffer, size_t len,
+                             sampleFormat format, ArrayOf<char> &cleanup)
 {
-   char* localFullSummary = new char[mSummaryInfo.totalSummaryBytes];
+   cleanup.reinit(mSummaryInfo.totalSummaryBytes);
+   char* localFullSummary = cleanup.get();
 
    memcpy(localFullSummary, bheaderTag, bheaderTagLen);
 
    float *summary64K = (float *)(localFullSummary + mSummaryInfo.offset64K);
    float *summary256 = (float *)(localFullSummary + mSummaryInfo.offset256);
 
+   Floats floats;
    float *fbuffer;
 
    //mchinen: think we can hack this - don't allocate and copy if we don't need to.,
@@ -412,105 +424,14 @@ void *ODDecodeBlockFile::CalcSummary(samplePtr buffer, sampleCount len,
    }
    else
    {
-      fbuffer = new float[len];
+      floats.reinit(len);
+      fbuffer = floats.get();
       CopySamples(buffer, format,
                (samplePtr)fbuffer, floatSample, len);
    }
-   sampleCount sumLen;
-   sampleCount i, j, jcount;
 
-   float min, max;
-   float sumsq;
+   BlockFile::CalcSummaryFromBuffer(fbuffer, len, summary256, summary64K);
 
-   // Recalc 256 summaries
-   sumLen = (len + 255) / 256;
-
-
-   for (i = 0; i < sumLen; i++) {
-      min = fbuffer[i * 256];
-      max = fbuffer[i * 256];
-      sumsq = ((float)min) * ((float)min);
-      jcount = 256;
-      if (i * 256 + jcount > len)
-         jcount = len - i * 256;
-      for (j = 1; j < jcount; j++) {
-         float f1 = fbuffer[i * 256 + j];
-         sumsq += ((float)f1) * ((float)f1);
-         if (f1 < min)
-            min = f1;
-         else if (f1 > max)
-            max = f1;
-      }
-
-      float rms = (float)sqrt(sumsq / jcount);
-
-      summary256[i * 3] = min;
-      summary256[i * 3 + 1] = max;
-      summary256[i * 3 + 2] = rms;
-   }
-
-   for (i = sumLen; i < mSummaryInfo.frames256; i++) {
-      // filling in the remaining bits with non-harming/contributing values
-      summary256[i * 3] = FLT_MAX;  // min
-      summary256[i * 3 + 1] = -FLT_MAX;   // max
-      summary256[i * 3 + 2] = 0.0f; // rms
-   }
-
-   // Recalc 64K summaries
-   sumLen = (len + 65535) / 65536;
-
-   for (i = 0; i < sumLen; i++) {
-      min = summary256[3 * i * 256];
-      max = summary256[3 * i * 256 + 1];
-      sumsq = (float)summary256[3 * i * 256 + 2];
-      sumsq *= sumsq;
-
-      for (j = 1; j < 256; j++) {
-         if (summary256[3 * (i * 256 + j)] < min)
-            min = summary256[3 * (i * 256 + j)];
-         if (summary256[3 * (i * 256 + j) + 1] > max)
-            max = summary256[3 * (i * 256 + j) + 1];
-         float r1 = summary256[3 * (i * 256 + j) + 2];
-         sumsq += r1*r1;
-      }
-
-      float rms = (float)sqrt(sumsq / 256);
-
-      summary64K[i * 3] = min;
-      summary64K[i * 3 + 1] = max;
-      summary64K[i * 3 + 2] = rms;
-   }
-   for (i = sumLen; i < mSummaryInfo.frames64K; i++) {
-      summary64K[i * 3] = 0.0f;
-      summary64K[i * 3 + 1] = 0.0f;
-      summary64K[i * 3 + 2] = 0.0f;
-   }
-
-   // Recalc block-level summary
-   min = summary64K[0];
-   max = summary64K[1];
-   sumsq = (float)summary64K[2];
-   sumsq *= sumsq;
-
-   for (i = 1; i < sumLen; i++) {
-      if (summary64K[3*i] < min)
-         min = summary64K[3*i];
-      else if (summary64K[3*i+1] > max)
-         max = summary64K[3*i+1];
-      float r1 = (float)summary64K[3*i+2];
-      sumsq += (r1*r1);
-   }
-
-   mMin = min;
-   mMax = max;
-   mRMS = sqrt(sumsq / sumLen);
-
-
-   //if we've used the float sample..
-   if(format!=floatSample)
-   {
-      delete[] fbuffer;
-   }
    return localFullSummary;
 }
 
@@ -524,37 +445,39 @@ void *ODDecodeBlockFile::CalcSummary(samplePtr buffer, sampleCount len,
 /// @param format The format to convert the data into
 /// @param start  The offset within the block to begin reading
 /// @param len    The number of samples to read
-int ODDecodeBlockFile::ReadData(samplePtr data, sampleFormat format,
-                                sampleCount start, sampleCount len)
+size_t ODDecodeBlockFile::ReadData(samplePtr data, sampleFormat format,
+                                size_t start, size_t len, bool mayThrow) const
 {
-   int ret;
-   LockRead();
+   auto locker = LockForRead();
    if(IsSummaryAvailable())
-      ret= SimpleBlockFile::ReadData(data,format,start,len);
+      return SimpleBlockFile::ReadData(data, format, start, len, mayThrow);
    else
    {
-      //we should do an ODRequest to start processing the data here, and wait till it finishes. and just do a SimpleBlockFIle
+      if (mayThrow)
+         throw NotYetAvailableException{ mAudioFileName };
+
+      //we should do an ODRequest to start processing the data here, and wait till it finishes. and just do a SimpleBlockFile
       //ReadData.
       ClearSamples(data, format, 0, len);
-      ret= len;
+      return 0;
    }
-   UnlockRead();
-   return ret;
 }
 
 /// Read the summary of this alias block from disk.  Since the audio data
 /// is elsewhere, this consists of reading the entire summary file.
+/// Fill with zeroes and return false if data are unavailable for any reason.
 ///
 /// @param *data The buffer where the summary data will be stored.  It must
 ///              be at least mSummaryInfo.totalSummaryBytes long.
-bool ODDecodeBlockFile::ReadSummary(void *data)
+bool ODDecodeBlockFile::ReadSummary(ArrayOf<char> &data)
 {
-   //I dont think we need to add a mutex here because only the main thread changes filenames and calls ReadSummarz
+   //I dont think we need to add a mutex here because only the main thread changes filenames and calls ReadSummary
    if(IsSummaryAvailable())
       return SimpleBlockFile::ReadSummary(data);
 
-   memset(data, 0, (size_t)mSummaryInfo.totalSummaryBytes);
-   return true;
+   data.reinit( mSummaryInfo.totalSummaryBytes );
+   memset(data.get(), 0, mSummaryInfo.totalSummaryBytes);
+   return false;
 }
 
 ///set the decoder,
@@ -570,12 +493,12 @@ void ODDecodeBlockFile::SetODFileDecoder(ODFileDecoder* decoder)
 
 
 /// Prevents a read on other threads.
-void ODDecodeBlockFile::LockRead()
+void ODDecodeBlockFile::LockRead() const
 {
    mReadDataMutex.Lock();
 }
 /// Allows reading of encoded file on other threads.
-void ODDecodeBlockFile::UnlockRead()
+void ODDecodeBlockFile::UnlockRead() const
 {
    mReadDataMutex.Unlock();
 }
@@ -583,9 +506,9 @@ void ODDecodeBlockFile::UnlockRead()
 /// Modify this block to point at a different file.  This is generally
 /// looked down on, but it is necessary in one case: see
 /// DirManager::EnsureSafeFilename().
-void ODDecodeBlockFile::ChangeAudioFile(wxFileName newAudioFile)
+void ODDecodeBlockFile::ChangeAudioFile(wxFileNameWrapper &&newAudioFile)
 {
-   mAudioFileName = newAudioFile;
+   mAudioFileName = std::move(newAudioFile);
 }
 
 
