@@ -18,17 +18,16 @@ and libvorbis examples, Monty <monty@xiph.org>
 
 **********************************************************************/
 
-#include "../Audacity.h"
+#include "../Audacity.h" // for USE_* macros
 
 #ifdef USE_LIBFLAC
-
 #include "ExportFLAC.h"
+
 #include "Export.h"
 
 #include <wx/progdlg.h>
 #include <wx/ffile.h>
 #include <wx/log.h>
-#include <wx/msgdlg.h>
 
 #include "FLAC++/encoder.h"
 
@@ -42,6 +41,8 @@ and libvorbis examples, Monty <monty@xiph.org>
 #include "../Tags.h"
 
 #include "../Track.h"
+#include "../widgets/ErrorDialog.h"
+#include "../widgets/ProgressDialog.h"
 
 //----------------------------------------------------------------------------
 // ExportFLACOptions Class
@@ -55,8 +56,8 @@ public:
    virtual ~ExportFLACOptions();
 
    void PopulateOrExchange(ShuttleGui & S);
-   bool TransferDataToWindow();
-   bool TransferDataFromWindow();
+   bool TransferDataToWindow() override;
+   bool TransferDataFromWindow() override;
 };
 
 ///
@@ -81,20 +82,39 @@ ExportFLACOptions::~ExportFLACOptions()
 ///
 void ExportFLACOptions::PopulateOrExchange(ShuttleGui & S)
 {
-   wxArrayString flacLevelNames, flacLevelLabels;
-   flacLevelLabels.Add(wxT("0")); flacLevelNames.Add(_("0 (fastest)"));
-   flacLevelLabels.Add(wxT("1")); flacLevelNames.Add(_("1"));
-   flacLevelLabels.Add(wxT("2")); flacLevelNames.Add(_("2"));
-   flacLevelLabels.Add(wxT("3")); flacLevelNames.Add(_("3"));
-   flacLevelLabels.Add(wxT("4")); flacLevelNames.Add(_("4"));
-   flacLevelLabels.Add(wxT("5")); flacLevelNames.Add(_("5"));
-   flacLevelLabels.Add(wxT("6")); flacLevelNames.Add(_("6"));
-   flacLevelLabels.Add(wxT("7")); flacLevelNames.Add(_("7"));
-   flacLevelLabels.Add(wxT("8")); flacLevelNames.Add(_("8 (best)"));
+   wxArrayStringEx flacLevelLabels{
+      wxT("0") ,
+      wxT("1") ,
+      wxT("2") ,
+      wxT("3") ,
+      wxT("4") ,
+      wxT("5") ,
+      wxT("6") ,
+      wxT("7") ,
+      wxT("8") ,
+   };
 
-   wxArrayString flacBitDepthNames, flacBitDepthLabels;
-   flacBitDepthLabels.Add(wxT("16")); flacBitDepthNames.Add(_("16 bit"));
-   flacBitDepthLabels.Add(wxT("24")); flacBitDepthNames.Add(_("24 bit"));
+   wxArrayStringEx flacLevelNames{
+      _("0 (fastest)") ,
+      _("1") ,
+      _("2") ,
+      _("3") ,
+      _("4") ,
+      _("5") ,
+      _("6") ,
+      _("7") ,
+      _("8 (best)") ,
+   };
+
+   wxArrayStringEx flacBitDepthLabels{
+      wxT("16") ,
+      wxT("24") ,
+   };
+
+   wxArrayStringEx flacBitDepthNames{
+      _("16 bit") ,
+      _("24 bit") ,
+   };
 
    S.StartVerticalLay();
    {
@@ -139,7 +159,7 @@ bool ExportFLACOptions::TransferDataFromWindow()
 // ExportFLAC Class
 //----------------------------------------------------------------------------
 
-#define SAMPLES_PER_RUN 8192
+#define SAMPLES_PER_RUN 8192u
 
 /* FLACPP_API_VERSION_CURRENT is 6 for libFLAC++ from flac-1.1.3 (see <FLAC++/export.h>) */
 #if !defined FLACPP_API_VERSION_CURRENT || FLACPP_API_VERSION_CURRENT < 6
@@ -173,6 +193,14 @@ static struct
 
 //----------------------------------------------------------------------------
 
+struct FLAC__StreamMetadataDeleter {
+   void operator () (FLAC__StreamMetadata *p) const
+   { if (p) ::FLAC__metadata_object_delete(p); }
+};
+using FLAC__StreamMetadataHandle = std::unique_ptr<
+   FLAC__StreamMetadata, FLAC__StreamMetadataDeleter
+>;
+
 class ExportFLAC final : public ExportPlugin
 {
 public:
@@ -181,8 +209,9 @@ public:
 
    // Required
 
-   wxWindow *OptionsCreate(wxWindow *parent, int format);
-   int Export(AudacityProject *project,
+   wxWindow *OptionsCreate(wxWindow *parent, int format) override;
+   ProgressResult Export(AudacityProject *project,
+               std::unique_ptr<ProgressDialog> &pDialog,
                unsigned channels,
                const wxString &fName,
                bool selectedOnly,
@@ -196,7 +225,8 @@ private:
 
    bool GetMetadata(AudacityProject *project, const Tags *tags);
 
-   FLAC__StreamMetadata *mMetadata;
+   // Should this be a stack variable instead in Export?
+   FLAC__StreamMetadataHandle mMetadata;
 };
 
 //----------------------------------------------------------------------------
@@ -212,7 +242,8 @@ ExportFLAC::ExportFLAC()
    SetDescription(_("FLAC Files"),0);
 }
 
-int ExportFLAC::Export(AudacityProject *project,
+ProgressResult ExportFLAC::Export(AudacityProject *project,
+                        std::unique_ptr<ProgressDialog> &pDialog,
                         unsigned numChannels,
                         const wxString &fName,
                         bool selectionOnly,
@@ -226,7 +257,7 @@ int ExportFLAC::Export(AudacityProject *project,
    const TrackList *tracks = project->GetTracks();
 
    wxLogNull logNo;            // temporarily disable wxWidgets error messages
-   int updateResult = eProgressSuccess;
+   auto updateResult = ProgressResult::Success;
 
    int levelPref;
    gPrefs->Read(wxT("/FileFormats/FLACLevel"), &levelPref, 5);
@@ -236,57 +267,81 @@ int ExportFLAC::Export(AudacityProject *project,
 
    FLAC::Encoder::File encoder;
 
+   bool success = true;
+   success = success &&
 #ifdef LEGACY_FLAC
-   encoder.set_filename(OSOUTPUT(fName));
+   encoder.set_filename(OSOUTPUT(fName)) &&
 #endif
-   encoder.set_channels(numChannels);
+   encoder.set_channels(numChannels) &&
    encoder.set_sample_rate(lrint(rate));
 
    // See note in GetMetadata() about a bug in libflac++ 1.1.2
-   if (!GetMetadata(project, metadata)) {
-      return false;
+   if (success && !GetMetadata(project, metadata)) {
+      // TODO: more precise message
+      AudacityMessageBox(_("Unable to export"));
+      return ProgressResult::Cancelled;
    }
 
-   if (mMetadata) {
-      encoder.set_metadata(&mMetadata, 1);
+   if (success && mMetadata) {
+      // set_metadata expects an array of pointers to metadata and a size.
+      // The size is 1.
+      FLAC__StreamMetadata *p = mMetadata.get();
+      success = encoder.set_metadata(&p, 1);
    }
 
+   auto cleanup1 = finally( [&] {
+      mMetadata.reset(); // need this?
+   } );
+
    sampleFormat format;
    if (bitDepthPref == wxT("24")) {
       format = int24Sample;
-      encoder.set_bits_per_sample(24);
+      success = success && encoder.set_bits_per_sample(24);
    } else { //convert float to 16 bits
       format = int16Sample;
-      encoder.set_bits_per_sample(16);
+      success = success && encoder.set_bits_per_sample(16);
    }
 
+
    // Duplicate the flac command line compression levels
    if (levelPref < 0 || levelPref > 8) {
       levelPref = 5;
    }
-   encoder.set_do_exhaustive_model_search(flacLevels[levelPref].do_exhaustive_model_search);
+   success = success &&
+   encoder.set_do_exhaustive_model_search(flacLevels[levelPref].do_exhaustive_model_search) &&
    encoder.set_do_escape_coding(flacLevels[levelPref].do_escape_coding);
+
    if (numChannels != 2) {
-      encoder.set_do_mid_side_stereo(false);
+      success = success &&
+      encoder.set_do_mid_side_stereo(false) &&
       encoder.set_loose_mid_side_stereo(false);
    }
    else {
-      encoder.set_do_mid_side_stereo(flacLevels[levelPref].do_mid_side_stereo);
+      success = success &&
+      encoder.set_do_mid_side_stereo(flacLevels[levelPref].do_mid_side_stereo) &&
       encoder.set_loose_mid_side_stereo(flacLevels[levelPref].loose_mid_side_stereo);
    }
-   encoder.set_qlp_coeff_precision(flacLevels[levelPref].qlp_coeff_precision);
-   encoder.set_min_residual_partition_order(flacLevels[levelPref].min_residual_partition_order);
-   encoder.set_max_residual_partition_order(flacLevels[levelPref].max_residual_partition_order);
-   encoder.set_rice_parameter_search_dist(flacLevels[levelPref].rice_parameter_search_dist);
+
+   success = success &&
+   encoder.set_qlp_coeff_precision(flacLevels[levelPref].qlp_coeff_precision) &&
+   encoder.set_min_residual_partition_order(flacLevels[levelPref].min_residual_partition_order) &&
+   encoder.set_max_residual_partition_order(flacLevels[levelPref].max_residual_partition_order) &&
+   encoder.set_rice_parameter_search_dist(flacLevels[levelPref].rice_parameter_search_dist) &&
    encoder.set_max_lpc_order(flacLevels[levelPref].max_lpc_order);
 
+   if (!success) {
+      // TODO: more precise message
+      AudacityMessageBox(_("Unable to export"));
+      return ProgressResult::Cancelled;
+   }
+
 #ifdef LEGACY_FLAC
    encoder.init();
 #else
    wxFFile f;     // will be closed when it goes out of scope
    if (!f.Open(fName, wxT("w+b"))) {
-      wxMessageBox(wxString::Format(_("FLAC export couldn't open %s"), fName.c_str()));
-      return false;
+      AudacityMessageBox(wxString::Format(_("FLAC export couldn't open %s"), fName));
+      return ProgressResult::Cancelled;
    }
 
    // Even though there is an init() method that takes a filename, use the one that
@@ -294,14 +349,22 @@ int ExportFLAC::Export(AudacityProject *project,
    // libflac can't (under Windows).
    int status = encoder.init(f.fp());
    if (status != FLAC__STREAM_ENCODER_INIT_STATUS_OK) {
-      wxMessageBox(wxString::Format(_("FLAC encoder failed to initialize\nStatus: %d"), status));
-      return false;
+      AudacityMessageBox(wxString::Format(_("FLAC encoder failed to initialize\nStatus: %d"), status));
+      return ProgressResult::Cancelled;
    }
 #endif
 
-   if (mMetadata) {
-      ::FLAC__metadata_object_delete(mMetadata);
-   }
+   mMetadata.reset();
+
+   auto cleanup2 = finally( [&] {
+      if (!(updateResult == ProgressResult::Success ||
+            updateResult == ProgressResult::Stopped)) {
+#ifndef LEGACY_FLAC
+         f.Detach(); // libflac closes the file
+#endif
+         encoder.finish();
+      }
+   } );
 
    const WaveTrackConstArray waveTracks =
       tracks->GetWaveTrackConstArray(selectionOnly, false);
@@ -311,51 +374,61 @@ int ExportFLAC::Export(AudacityProject *project,
                             numChannels, SAMPLES_PER_RUN, false,
                             rate, format, true, mixerSpec);
 
-   int i;
-   FLAC__int32 **tmpsmplbuf = new FLAC__int32*[numChannels];
-   for (i = 0; i < numChannels; i++) {
-      tmpsmplbuf[i] = (FLAC__int32 *) calloc(SAMPLES_PER_RUN, sizeof(FLAC__int32));
-   }
+   ArraysOf<FLAC__int32> tmpsmplbuf{ numChannels, SAMPLES_PER_RUN, true };
 
-   {
-      ProgressDialog progress(wxFileName(fName).GetName(),
-         selectionOnly ?
-         _("Exporting the selected audio as FLAC") :
-         _("Exporting the entire project as FLAC"));
+   InitProgress( pDialog, wxFileName(fName).GetName(),
+      selectionOnly
+         ? _("Exporting the selected audio as FLAC")
+         : _("Exporting the audio as FLAC") );
+   auto &progress = *pDialog;
 
-      while (updateResult == eProgressSuccess) {
-         auto samplesThisRun = mixer->Process(SAMPLES_PER_RUN);
-         if (samplesThisRun == 0) { //stop encoding
-            break;
-         }
-         else {
-            for (i = 0; i < numChannels; i++) {
-               samplePtr mixed = mixer->GetBuffer(i);
-               if (format == int24Sample) {
-                  for (decltype(samplesThisRun) j = 0; j < samplesThisRun; j++) {
-                     tmpsmplbuf[i][j] = ((int *)mixed)[j];
-                  }
+   while (updateResult == ProgressResult::Success) {
+      auto samplesThisRun = mixer->Process(SAMPLES_PER_RUN);
+      if (samplesThisRun == 0) { //stop encoding
+         break;
+      }
+      else {
+         for (size_t i = 0; i < numChannels; i++) {
+            samplePtr mixed = mixer->GetBuffer(i);
+            if (format == int24Sample) {
+               for (decltype(samplesThisRun) j = 0; j < samplesThisRun; j++) {
+                  tmpsmplbuf[i][j] = ((int *)mixed)[j];
                }
-               else {
-                  for (decltype(samplesThisRun) j = 0; j < samplesThisRun; j++) {
-                     tmpsmplbuf[i][j] = ((short *)mixed)[j];
-                  }
+            }
+            else {
+               for (decltype(samplesThisRun) j = 0; j < samplesThisRun; j++) {
+                  tmpsmplbuf[i][j] = ((short *)mixed)[j];
                }
             }
-            encoder.process(tmpsmplbuf, samplesThisRun);
          }
-         updateResult = progress.Update(mixer->MixGetCurrentTime() - t0, t1 - t0);
+         if (! encoder.process(
+               reinterpret_cast<FLAC__int32**>( tmpsmplbuf.get() ),
+               samplesThisRun) ) {
+            // TODO: more precise message
+            AudacityMessageBox(_("Unable to export"));
+            updateResult = ProgressResult::Cancelled;
+            break;
+         }
+         if (updateResult == ProgressResult::Success)
+            updateResult =
+               progress.Update(mixer->MixGetCurrentTime() - t0, t1 - t0);
       }
+   }
+
+   if (updateResult == ProgressResult::Success ||
+       updateResult == ProgressResult::Stopped) {
+#ifndef LEGACY_FLAC
       f.Detach(); // libflac closes the file
-      encoder.finish();
+#endif
+      if (!encoder.finish())
+         // Do not reassign updateResult, see cleanup2
+         return ProgressResult::Failed;
+#ifdef LEGACY_FLAC
+      if (!f.Flush() || !f.Close())
+         return ProgressResult::Failed;
+#endif
    }
 
-   for (i = 0; i < numChannels; i++) {
-      free(tmpsmplbuf[i]);
-   }
-
-   delete[] tmpsmplbuf;
-
    return updateResult;
 }
 
@@ -377,7 +450,7 @@ bool ExportFLAC::GetMetadata(AudacityProject *project, const Tags *tags)
    if (tags == NULL)
       tags = project->GetTags();
 
-   mMetadata = ::FLAC__metadata_object_new(FLAC__METADATA_TYPE_VORBIS_COMMENT);
+   mMetadata.reset(::FLAC__metadata_object_new(FLAC__METADATA_TYPE_VORBIS_COMMENT));
 
    wxString n;
    for (const auto &pair : tags->GetRange()) {
@@ -388,17 +461,18 @@ bool ExportFLAC::GetMetadata(AudacityProject *project, const Tags *tags)
       }
       FLAC::Metadata::VorbisComment::Entry entry(n.mb_str(wxConvUTF8),
                                                  v.mb_str(wxConvUTF8));
-      ::FLAC__metadata_object_vorbiscomment_append_comment(mMetadata,
+      if (! ::FLAC__metadata_object_vorbiscomment_append_comment(mMetadata.get(),
                                                            entry.get_entry(),
-                                                           true);
+                                                           true) )
+         return false;
    }
 
    return true;
 }
 
-movable_ptr<ExportPlugin> New_ExportFLAC()
+std::unique_ptr<ExportPlugin> New_ExportFLAC()
 {
-   return make_movable<ExportFLAC>();
+   return std::make_unique<ExportFLAC>();
 }
 
 #endif // USE_LIBFLAC
